@@ -5,5 +5,5 @@
 # Copyright 2022 Oxide Computer Company
 
 from duro_rest.error import AuthenticationError, BadRequestError, ForbiddenError, NotFoundError
-from duro_rest.duro_client import Client
-from duro_rest.bom_client import BOMClient
\ No newline at end of file
+from duro_rest.duro_client import Client, Cache, FileCache
+from duro_rest.bom_client import BOMClient, NestedBOM, IndentedBOM, FlattenedBOM
\ No newline at end of file
