@@ -4,16 +4,64 @@
 
 # Copyright 2022 Oxide Computer Company
 
-from datetime import datetime
+from datetime import datetime, timedelta
+import json
+from json import JSONDecodeError
 import math
 import requests
 
 from duro_rest.error import AuthenticationError, BadRequestError, ForbiddenError, NotFoundError
 
+class Cache():
+  def __init__(self, cache_duration):
+    self.__delta = timedelta(seconds = cache_duration)
+    self.__data = {}
+
+  def get(self, key):
+    if key in self.__data:
+      (value, expiration) = self.__data[key]
+
+      if datetime.now().timestamp() < expiration:
+        return value
+
+    return None
+
+  def insert(self, key, value):
+    if self.__delta.seconds > 0:
+      self.__data[key] = (value, (datetime.now() + self.__delta).timestamp())
+
+class FileCache(Cache):
+  def __init__(self, cache_duration, path):
+    super().__init__(cache_duration)
+
+    try:
+      self.__cache_file = open(path, "r+")
+      stored = json.load(self.__cache_file)
+
+      if stored != None:
+        self._Cache__data = stored
+    except TypeError:
+      # Ignore
+      pass
+    except JSONDecodeError:
+      # Ignore
+      pass
+    except FileNotFoundError:
+      self.__cache_file = open(path, "w")
+      pass
+    finally:
+      # Make sure we always rewind to the start of the file
+      self.__cache_file.seek(0, 0)
+
+  def save(self):
+    self.__cache_file.write(json.dumps(self._Cache__data))
+    self.__cache_file.seek(0, 0)
+
 class Client:
-  def __init__(self, api_key, url = "https://public-api.duro.app/v1/"):
-    self.__base_url = url
+  def __init__(self, api_key, url = "https://public-api.duro.app", cache = Cache(0)):
+    self.__base_url = url + "/v1/"
     self.__api_key = api_key
+    self.__cache = cache
 
   def categories(
     self,
@@ -32,7 +80,7 @@ class Client:
     )
 
   def change_order(self, id):
-    return self.__get("changeorders/" + id).json()
+    return self.__get("changeorders/" + id)
 
   def change_orders(
     self,
@@ -53,7 +101,14 @@ class Client:
     )
 
   def component(self, id):
-    return self.__get("components/" + id).json()
+
+    # Try to serve from the cache
+    if entries := self.__cache.get("components"):
+      for entry in entries:
+        if entry["_id"] == id:
+          return entry
+
+    return self.__get("components/" + id)
 
   def components(
     self,
@@ -96,7 +151,7 @@ class Client:
     )
 
   def component_revision(self, id):
-    return self.__get("component/revision/" + id).json()
+    return self.__get("component/revision/" + id)
 
   def component_revisions(
     self,
@@ -139,10 +194,10 @@ class Client:
     )
 
   def document(self, id):
-    return self.__get("documents/" + id).json()
+    return self.__get("documents/" + id)
 
   def product(self, id):
-    return self.__get("products/" + id).json()
+    return self.__get("products/" + id)
 
   def products(
     self,
@@ -181,7 +236,7 @@ class Client:
     )
 
   def product_revision(self, id):
-    return self.__get("product/revision/" + id).json()
+    return self.__get("product/revision/" + id)
 
   def product_revisions(
     self,
@@ -220,7 +275,7 @@ class Client:
     )
 
   def user(self, id):
-    return self.__get("users/" + id).json()
+    return self.__get("users/" + id)
 
   def users(
     self,
@@ -239,7 +294,7 @@ class Client:
   def __get_paginated(self, path, resultKey, params):
     resources = []
 
-    page = self.__get(path, params = {"perPage": 100, **params}).json()
+    page = self.__get(path, params = {"perPage": 100, **params})
 
     total_results = page["resultCount"]
     pages = math.ceil(total_results / 100)
@@ -248,12 +303,21 @@ class Client:
 
     if pages > 1:
       for page_index in range(2, pages):
-        page = self.__get(path, params = {"perPage": 100, "page": page_index}).json()
+        page = self.__get(path, params = {**params, "perPage": 100, "page": page_index})
         resources = resources + page[resultKey]
 
+    # For each resource we found populate its individual record in the cache
+    for resource in resources:
+      self.__cache.insert(path + "/" + resource["_id"] + "::{}", resource)
+
     return resources
 
   def __get(self, path, headers = {}, params = {}, data = None):
+    cache_key = path + "::" + json.dumps(params)
+
+    if (value := self.__cache.get(cache_key)) is not None:
+      return value
+
     headers["x-api-key"] = self.__api_key
 
     resp = requests.get(self.__base_url + path, headers = headers, params = params, data = data)
@@ -267,4 +331,6 @@ class Client:
     if resp.status_code == 404:
       raise NotFoundError(resp.json()["message"])
     else:
-      return resp
\ No newline at end of file
+      data = resp.json()
+      self.__cache.insert(cache_key, data)
+      return data
\ No newline at end of file
